# -*- coding: utf-8 -*-
from django import apps
from django.utils.translation import gettext_lazy as _


class AppConfig(apps.AppConfig):
	name = 'django_ajax_utils'
	verbose_name = _('Django ajax utils')



import os

BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))

SECRET_KEY = 'secret_key'

DEBUG = True

ALLOWED_HOSTS = []

INSTALLED_APPS = [
	#'django.contrib.auth',
	#'django.contrib.contenttypes',
	'django.contrib.sessions',
	'django.contrib.messages',
	'django.contrib.staticfiles',
	'django_ajax_utils',
]

MIDDLEWARE = [
	#'django.middleware.security.SecurityMiddleware',
	'django.contrib.sessions.middleware.SessionMiddleware',
	#'django.middleware.common.CommonMiddleware',
	#'django.middleware.csrf.CsrfViewMiddleware',
	#'django.contrib.auth.middleware.AuthenticationMiddleware',
	'django.contrib.messages.middleware.MessageMiddleware',
	#'django.middleware.clickjacking.XFrameOptionsMiddleware',
	'django_ajax_utils.middleware.RemoveDummyParametersMiddleware',
	'django_ajax_utils.pjax.Middleware',
]

ROOT_URLCONF = 'web.urls'

TEMPLATES = [
	{
		'BACKEND': 'django.template.backends.django.DjangoTemplates',
		'DIRS': [os.path.join(BASE_DIR, 'templates'),],
		'OPTIONS': {
			'context_processors': [
				#'django.template.context_processors.debug',
				'django.template.context_processors.request',
				#'django.contrib.auth.context_processors.auth',
				'django.contrib.messages.context_processors.messages',
			],
			'loaders': [
				'django_ajax_utils.pjax.Loader',
				'django.template.loaders.filesystem.Loader',
				'django.template.loaders.app_directories.Loader',
			],
		},
	},
]

WSGI_APPLICATION = 'web.wsgi.application'


DATABASES = {
	'default': {
		'ENGINE': 'django.db.backends.sqlite3',
		'NAME': os.path.join(BASE_DIR, 'db.sqlite3'),
	}
}

PJAX_INCLUDE_URLPATTERNS = [r'.*']
PJAX_EXCLUDE_URLPATTERNS = [r'^admin:.*$']

LANGUAGE_CODE = 'en-us'

TIME_ZONE = 'UTC'

USE_I18N = False

USE_L10N = False

USE_TZ = False

STATIC_URL = '/static/'

STATICFILES_DIRS = (os.path.join(BASE_DIR, 'static'),)



# -*- coding: utf-8 -*-
from django import forms
from django.core.serializers.json import DjangoJSONEncoder
from django.http import JsonResponse
from django.http.response import HttpResponseRedirect

from .utility import check_json


class SafeDjangoJSONEncoder(DjangoJSONEncoder):
	def default(self, o):
		try:
			val = super().default(o)
		except TypeError:
			val = f'<not serializable: {type(o).__qualname__}>'
		return val


class JsonResponseMixin(object):
	"""
	Mixin that handles JSON serialization.
	"""

	def render_json_response(self, data, **kwargs):
		"""
		Renders data as JSON into HttpResponse.
		"""
		kwargs.setdefault('safe', False)
		return JsonResponse(data, **kwargs)


class AjaxRedirectMixin(JsonResponseMixin):
	"""
	Mixin that transforms HttpResponseRedirect to JSON response
	{"redirect": "url"} for ajax calls.
	"""

	def dispatch(self, request, *args, **kwargs):
		response = super().dispatch(request, *args, **kwargs)
		if check_json(request):
			if isinstance(response, HttpResponseRedirect):
				original_response = response
				url = original_response.url
				json_data = {"redirect": url}
				if getattr(response, 'plain_redirect', False):
					json_data['plain_redirect'] = True
				if hasattr(response, 'redirect_data'):
					json_data.update(response.redirect_data)
				response = self.render_json_response(json_data)
				response.cookies = original_response.cookies
		return response


class AjaxFormMixin(AjaxRedirectMixin, JsonResponseMixin):
	only_validate_field = 'only_validate'

	@property
	def only_validate_form(self):
		return bool(self.request.POST.get(self.only_validate_field, False))

	def post(self, request, *args, **kwargs):
		if check_json(request):
			response = super().post(request, *args, **kwargs)
			if hasattr(response, 'context_data'):
				return self.format_forms_status(response)
			else:
				return response
		else:
			return super().post(request, *args, **kwargs)

	def form_valid(self, form):
		if self.only_validate_form:
			return self.render_to_response(self.get_context_data(form=form))
		return super().form_valid(form)

	def format_forms_status(self, response, status_code=200):
		ctx = response.context_data
		json_response = {'forms': {}}
		for key, form in ctx.items():
			if isinstance(form, (forms.BaseForm, forms.BaseFormSet)) and form.is_bound:
				json_response['forms'][key] = self.format_form_status(form)
		return self.render_json_response(json_response, status=status_code, encoder=SafeDjangoJSONEncoder)

	def format_form_status(self, form):
		form_data = {
			'status': 'valid' if form.is_valid() else 'invalid',
			'errors': {},
			'valid': [],
			'empty': [],
			'changed': [],
			'prefix': form.prefix,
		}

		def add_formset_status(formset):
			total_form_count = 0
			for formrow in formset:
				total_form_count += 1
				if formset.can_delete and formset._should_delete_form(formrow):
					for name, __ in formrow.fields.items():
						form_data['empty'].append(formrow.add_prefix(name))
				else:
					status = self.format_form_status(formrow)
					form_data['errors'].update(status['errors'])
					form_data['valid'] += status['valid']
					form_data['empty'] += status['empty']
			form_data['total_form_count'] = total_form_count

		if not form.is_valid():
			if hasattr(form, 'non_form_errors'):
				errors = self.__format_errorlist(form.non_form_errors())
				if errors:
					form_data['errors'][form.add_prefix('__all__')] = errors
			if isinstance(form, forms.BaseFormSet):
				add_formset_status(form)
				return form_data
			else:
				for field, errors in form.errors.items():
					fieldname = form.add_prefix(field)
					form_data['errors'][fieldname] = self.__format_errorlist(errors)

		if isinstance(form, forms.BaseFormSet):
			add_formset_status(form)
		else:
			form_data.update(self.__format_fields_status(form, form_data))
		return form_data

	def __format_errorlist(self, errorlist):
		errors = []
		for error in errorlist.as_data():
			message = next(iter(error))
			error_data = {
				'message': message,
				'code': error.code or '',
			}
			if getattr(error, 'params', None):
				error_data['params'] = error.params
			errors.append(error_data)
		return errors

	def __format_fields_status(self, form, form_data):
		valid = []
		empty = []
		if hasattr(form, 'cleaned_data'):
			for name, field in form.fields.items():
				fieldname = form.add_prefix(name)
				if field.widget.value_from_datadict(form.data, form.files, form.add_prefix(name)) in field.empty_values:
					empty.append(fieldname)
				if fieldname in form_data['errors'] or name not in form.cleaned_data or form.cleaned_data[name] in field.empty_values:
					continue
				valid.append(fieldname)
		return {
			'valid': valid,
			'empty': empty,
			'changed': form.changed_data,
		}



# -*- coding: utf-8 -*-
from django.forms import CheckboxInput
from django.utils.encoding import force_str

from .settings import FORMUTILS_ROW_TEMPLATE_ATTRIBUTE


class AutoPlaceholderFormMixin(object):
	"""
	Adds placeholder attributes to form inputs.
	"""

	def __init__(self, *args, **kwargs):
		super(AutoPlaceholderFormMixin, self).__init__(*args, **kwargs)
		for field in self.fields.values():
			if not 'placeholder' in field.widget.attrs and field.label:
				placeholder = field.help_text or field.label
				label = force_str(field.label)
				if not hasattr(label, '__html__') and not isinstance(field.widget, CheckboxInput):
					field.widget.attrs['placeholder'] = placeholder


class SetRequiredFieldsMixin(object):
	"""
	Overrides required attribute for selected fields.

	Example

	class Foo(SetRequiredFieldsMixin, forms.ModelForm):
		required_fields = {
			'field1': True,
			'field2': False,
		}
	"""

	required_fields = {}

	def __init__(self, *args, **kwargs):
		super(SetRequiredFieldsMixin, self).__init__(*args, **kwargs)
		for fieldname, is_required in self.required_fields.items():
			self.set_field_required(fieldname, is_required)

	def set_field_required(self, fieldname, is_required):
		self.fields[fieldname].required = is_required


class SetWidgetAttrsMixin(object):
	"""
	Overrides widget attributes.

	Example

	class Foo(SetWidgetAttrsMixin, forms.ModelForm):
		widget_attrs = {
			'field1': {'maxlength': '10'},
		}
	"""

	widget_attrs = {}

	def __init__(self, *args, **kwargs):
		super(SetWidgetAttrsMixin, self).__init__(*args, **kwargs)
		for fieldname, attrs in self.widget_attrs.items():
			if fieldname in self.fields:
				self.fields[fieldname].widget.attrs.update(attrs)


try:
	from jinja2 import nodes
	from jinja2.ext import Extension
	from jinja2.exceptions import TemplateSyntaxError


	class FormExtension(Extension):
		tags = set(['form', 'formrow', 'formrow_template'])

		def parse(self, parser):
			tag = next(parser.stream)
			if tag.value == 'form':
				return self.parse_form(parser, tag)
			elif tag.value == 'formrow':
				return self.parse_formrow(parser, tag)
			elif tag.value == 'formrow_template':
				return self.parse_formrow_template(parser, tag)

		def parse_form(self, parser, tag):
			lineno = tag.lineno
			form_instance = parser.parse_expression()
			template_name = nodes.Call(
				nodes.Name('get_formlayout_template', 'load'),
				[],
				[
					nodes.Keyword('caller_template', nodes.Const(parser.name)),
					nodes.Keyword('form', form_instance),
				],
				None,
				None,
			)
			has_body = False
			if not parser.stream.current.test('block_end'):
				parser.stream.expect('name:using')
				if parser.stream.current.test('block_end'):
					has_body = True
			if not parser.stream.current.test('block_end'):
				template_name = parser.parse_expression()
			if not parser.stream.current.test('block_end'):
				raise TemplateSyntaxError("Too many arguments", lineno)

			body = None
			if has_body:
				body = parser.parse_statements(['name:endform'], drop_needle=True)
			else:
				body = nodes.Include(template_name, True, False)
				body = [body]

			node = nodes.Scope(lineno=lineno)
			assignments = [
				nodes.Assign(nodes.Name('form_utils_form', 'store'), form_instance)
			]
			node.body = assignments + body
			return node.set_lineno(lineno)

		def parse_formrow(self, parser, tag):
			lineno = tag.lineno
			field = parser.parse_expression()
			template_name = None
			if not parser.stream.current.test('block_end'):
				template_name = parser.parse_expression()
			else:
				template_name = nodes.Call(
					nodes.Name('get_formrow_template', 'load'),
					[],
					[
						nodes.Keyword('caller_template', nodes.Const(parser.name)),
						nodes.Keyword('form', nodes.Name('form_utils_form', 'load'))
					],
					None,
					None,
				)
			if not parser.stream.current.test('block_end'):
				raise TemplateSyntaxError("Too many arguments", lineno)

			node = nodes.Scope(lineno=lineno)
			assignments = [
				nodes.Assign(nodes.Name('field', 'store'), field)
			]
			node.body = assignments + [nodes.Include(template_name, True, False)]
			return node.set_lineno(lineno)

		def parse_formrow_template(self, parser, tag):
			lineno = tag.lineno
			template_name = parser.parse_expression()
			if not parser.stream.current.test('block_end'):
				raise TemplateSyntaxError("Too many arguments", lineno)
			call = self.call_method('_process', [template_name, nodes.Name('form_utils_form', 'load', lineno=lineno)])
			return nodes.Output([nodes.MarkSafe(call)]).set_lineno(lineno)

		def _process(self, template_name, form_instance):
			setattr(form_instance, FORMUTILS_ROW_TEMPLATE_ATTRIBUTE, template_name)
			return ''


except ImportError:
	pass



# -*- coding: utf-8 -*-
import os
from collections import namedtuple
from dataclasses import dataclass, field
from typing import Optional, Union


def template_name_add_suffix(template_name, suffix):
	"""
	Add suffix to template name. Example

	template_name = "home.html", suffix = "ajax" -> returns "home_ajax.html"
	"""
	return suffix.join(os.path.splitext(template_name))


def template_names_add_suffix(template_names, suffix):
	"""
	Add suffix to each template name from list template_names
	"""
	if isinstance(template_names, str):
		template_names  = (template_names,)
	return tuple(template_name_add_suffix(tpl, suffix) for tpl in template_names)


def get_accept_parser(request):
	"""
	Returns AcceptParser for request
	"""
	if not hasattr(request, 'accept_parser'):
		request.accept_parser = AcceptParser(request.META.get('HTTP_ACCEPT'))
	return request.accept_parser


JSON_SUPPORTED_TYPES = ['application/json', 'application/pjax.json']


def check_json(request):
	"""
	Returns True if request is from ajax call
	"""
	mimetype = get_accept_parser(request).negotiate(JSON_SUPPORTED_TYPES, wildcard=False)
	return bool(mimetype)


MimeType = namedtuple('MimeType', ['content_type', 'content_subtype'])


@dataclass
class ContentType(object):
	content_type: str = '*'
	content_subtype: str = '*'
	quality: float = 1.0
	parameters: dict = field(default_factory=dict)

	def __repr__(self):
		return f'{self.content_type}/{self.content_subtype}; q={self.quality}' + ''.join(f'; {key}=val' for key, val in self.parameters.items())

	@staticmethod
	def parse(accept: str) -> 'ContentType':
		params = [part.strip() for part in accept.split(';') if part.strip()]
		if not params:
			return ContentType()
		q = 1.0
		content_type, params = params[0], params[1:]
		try:
			content_type, content_subtype = content_type.split('/', 1)
		except ValueError:
			content_subtype = '*'
		parameters = {}
		for param in params:
			try:
				param, value = param.split('=', 1)
			except ValueError:
				value = ''
			param = param.lower()
			if param == 'q':
				try:
					q = float(value)
				except ValueError:
					q = 1.0
			else:
				parameters[param] = value
		return ContentType(content_type, content_subtype, q, parameters)

	def matches(self, other: 'ContentType', wildcard: bool = True) -> bool:
		if not wildcard:
			return self.content_type == other.content_type and self.content_subtype == other.content_subtype
		if self.content_type != '*' and other.content_type != '*' and self.content_type != other.content_type:
			return False
		if self.content_subtype != '*' and other.content_subtype != '*' and self.content_subtype != other.content_subtype:
			return False
		if self.parameters and other.parameters and self.parameters != other.parameters:
			return False
		return True

	@property
	def full_type(self):
		return f'{self.content_type}/{self.content_subtype}'


class AcceptParser(object):
	__slots__ = ['__content_types', '__mimetypes', 'main_mimetype']

	def __init__(self, accept):
		if accept:
			self.__content_types = [ContentType.parse(part.strip()) for part in accept.split(',') if part.strip()]
		else:
			self.__content_types = []
		self.__content_types = sorted(self.__content_types, key=lambda ct: -ct.quality)
		self.__mimetypes = set(f'{ct.content_type}/{ct.content_subtype}' for ct in self.__content_types)

		if self.__content_types:
			main_content_type = self.__content_types[0]
			self.main_mimetype = f'{main_content_type.content_type}/{main_content_type.content_subtype}'
		else:
			self.main_mimetype = ''

	def __repr__(self):
		return ', '.join(repr(content_type) for content_type in self.__content_types)

	def __split_content_type(self, content_type: str) -> MimeType:
		try:
			content_type, content_subtype = content_type.split('/', 1)
		except ValueError:
			content_subtype = '*'
		return MimeType(content_type, content_subtype)

	def __query_to_content_type(self, query: list) -> list:
		choices = []
		for choice in query:
			if isinstance(choice, ContentType):
				choices.append(choice)
			elif isinstance(choice, str):
				choices.append(ContentType(*self.__split_content_type(choice)))
			elif isinstance(choice, (list, tuple)):
				mtype = self.__split_content_type(choice[0])
				choices.append(ContentType(mtype.content_type, mtype.content_subtype, 1.0, choice[1]))
			else:
				raise ValueError("Not supported choice type")
		return choices

	def negotiate(self, query: Union[list, str], wildcard: bool = True) -> Optional[ContentType]:
		"""
		Find best matching content type
		"""
		if isinstance(query, str):
			query = [query]
		query = self.__query_to_content_type(query)
		for content_type in self.__content_types:
			for choice in query:
				if content_type.matches(choice, wildcard):
					return content_type

	def has_content_type(self, content_type: str) -> bool:
		"""
		Simple check if content type is defined in accept. This will not match
		mime type wildcards.
		"""
		return content_type in self.__mimetypes



# -*- coding: utf-8 -*-
import types
from importlib import import_module

from django import forms, template

from ..settings import FORMUTILS_ROW_TEMPLATE_ATTRIBUTE, FORMUTILS_DEFAULT_LAYOUT_TEMPLATE, FORMUTILS_DEFAULT_ROW_TEMPLATE, FORMUTILS_DEFAULT_ROW_TEMPLATE_GET, FORMUTILS_DEFAULT_LAYOUT_TEMPLATE_GET


register = template.Library()


class FormNode(template.Node):
	template_name = None

	def __init__(self, form_instance, template_name, nodelist):
		self.form = template.Variable(form_instance)
		self.empty_nodelist = nodelist is None
		self.nodelist = nodelist or []
		if template_name is not None:
			self.template_name = template.Variable(template_name)

	def render(self, context):
		context.push()
		try:
			form_instance = self.form.resolve(context)
			setattr(form_instance, FORMUTILS_ROW_TEMPLATE_ATTRIBUTE, 'form_utils/row/default.html')
			context['form_utils_form'] = form_instance
			context['form'] = form_instance
			if self.empty_nodelist:
				template_name = self.template_name.resolve(context)
				t = context.template.engine.get_template(template_name)
				output = t.render(context)
			else:
				output = self.nodelist.render(context)
		finally:
			context.pop()
		return output


def form(parser, token):
	contents = token.split_contents()
	tag_name = contents[0]
	if len(contents) < 2:
		raise template.TemplateSyntaxError('%r tag requires form argument' % tag_name)
	if len(contents) > 4:
		raise template.TemplateSyntaxError('%r tag requires max 3 arguments' % tag_name)
	form_instance = contents[1]
	template_name = '"form_utils/layout/default.html"'
	nodelist = None
	if len(contents) > 2:
		if contents[2] != 'using':
			raise template.TemplateSyntaxError('%r tag\'s first argument should be using' % tag_name)
		if len(contents) == 3:
			nodelist = parser.parse(('end%s' % tag_name,))
			parser.delete_first_token()
		else:
			template_name = contents[3]
	return FormNode(form_instance, template_name, nodelist)


register.tag(form)


@register.simple_tag(takes_context=True)
def formrow_template(context, template_name):
	if not 'form_utils_form' in context:
		raise template.TemplateSyntaxError('This tag is allowed only inside template form tag')
	setattr(context['form_utils_form'], FORMUTILS_ROW_TEMPLATE_ATTRIBUTE, template_name)
	return ''


@register.simple_tag(takes_context=True)
def formrow(context, field, template_name=None):
	if not 'form_utils_form' in context:
		raise template.TemplateSyntaxError('This tag is allowed only inside template form tag')
	if not isinstance(field, (forms.Field, forms.BoundField)):
		return ''
	if template_name is None:
		template_name = getattr(context['form_utils_form'], FORMUTILS_ROW_TEMPLATE_ATTRIBUTE)
	context.push()
	try:
		context['field'] = field
		t = context.template.engine.get_template(template_name)
		output = t.render(context)
	finally:
		context.pop()
	return output


FORMUTILS_DEFAULT_LAYOUT_TEMPLATE = 'form_utils/layout/default.jinja' if FORMUTILS_DEFAULT_LAYOUT_TEMPLATE is None else FORMUTILS_DEFAULT_LAYOUT_TEMPLATE
FORMUTILS_DEFAULT_ROW_TEMPLATE = 'form_utils/row/default.jinja' if FORMUTILS_DEFAULT_ROW_TEMPLATE is None else FORMUTILS_DEFAULT_ROW_TEMPLATE


def import_name(path):
	module_name, obj_name = path.rsplit('.', 1)
	module = import_module(module_name)
	return getattr(module, obj_name)


if FORMUTILS_DEFAULT_LAYOUT_TEMPLATE_GET is None:
	def get_formlayout_template(form, caller_template, **kwargs): # pylint: disable=unused-argument
		return FORMUTILS_DEFAULT_LAYOUT_TEMPLATE
else:
	get_formlayout_template = import_name(FORMUTILS_DEFAULT_LAYOUT_TEMPLATE_GET)


if FORMUTILS_DEFAULT_ROW_TEMPLATE_GET is None:
	def get_formrow_template(form, caller_template, **kwargs): # pylint: disable=unused-argument
		return getattr(form, FORMUTILS_ROW_TEMPLATE_ATTRIBUTE, FORMUTILS_DEFAULT_ROW_TEMPLATE)
else:
	get_formrow_template = import_name(FORMUTILS_DEFAULT_ROW_TEMPLATE_GET)


jinja_get_formlayout_template = get_formlayout_template
jinja_get_formrow_template = get_formrow_template


class AsWidgetModifier(object):
	def __init__(self, field, original_as_widget):
		self.original_field = field
		self.original_as_widget = original_as_widget
		self.__additional_classes = set()
		self.__attributes = {}

	def __call__(self, widget=None, attrs=None, *args, **kwargs):
		if not widget:
			widget = self.original_field.widget
		attrs = attrs or widget.attrs.copy()
		if self.__additional_classes:
			cls = attrs.get('class', '').strip()
			cls += ' ' + ' '.join(self.__additional_classes)
			cls = cls.strip()
			attrs['class'] = cls
		attrs.update(self.__attributes)
		return self.original_as_widget(widget=widget, attrs=attrs, **kwargs)

	def __get__(self, instance, owner):
		return types.MethodType(self, instance) if instance else self

	def add_field_class(self, cls):
		if isinstance(cls, str):
			cls = cls.split()
		self.__additional_classes.update(cls)

	def add_attribute(self, key, value):
		self.__attributes[key] = value


def patch_as_widget(field):
	if not hasattr(field.as_widget, 'original_field'):
		field.as_widget = AsWidgetModifier(field.field, field.as_widget)
	return field.as_widget


@register.filter
def add_field_class(field, cls):
	as_widget_modifier = patch_as_widget(field)
	as_widget_modifier.add_field_class(cls)
	return field


@register.filter
def add_field_attr(field, attr):
	key, value = attr.split('=', 1)
	as_widget_modifier = patch_as_widget(field)
	as_widget_modifier.add_attribute(key, value)
	return field


@register.filter
def is_checkbox(field):
	return isinstance(field.field.widget, forms.CheckboxInput)


@register.filter
def is_select(field):
	return isinstance(field.field.widget, forms.Select)


@register.filter
def is_radio(field):
	return isinstance(field.field.widget, forms.RadioSelect)


@register.filter
def is_multiple(field):
	return isinstance(field.field, forms.MultipleChoiceField)


try:
	from django_jinja import library

	#library.global_function(contextfunction(formrow_template))
	library.global_function(fn=jinja_get_formlayout_template, name='get_formlayout_template')
	library.global_function(fn=jinja_get_formrow_template, name='get_formrow_template')
	library.filter(add_field_class)
	library.filter(is_checkbox)
	library.filter(is_select)
	library.filter(is_radio)
	library.filter(is_multiple)
	library.filter(add_field_attr)
except ImportError:
	pass



# -*- coding: utf-8 -*-
import json

from django.core.serializers.json import DjangoJSONEncoder
from django.db import models
from django.utils.translation import gettext_lazy as _


class JSONField(models.TextField):
	description = _("JSON data")

	def from_db_value(self, value, *args, **kwargs):
		if value is None:
			return
		try:
			return json.loads(value)
		except ValueError:
			pass

	def to_python(self, value):
		if isinstance(value, str):
			return json.loads(value)
		return value

	def get_db_prep_value(self, value, connection=None, prepared=None):
		return self.get_prep_value(value)

	def get_prep_value(self, value):
		if value == '':
			return ''
		if value is None and self.null:
			return
		if not isinstance(value, str):
			return json.dumps(value, cls=DjangoJSONEncoder, sort_keys=True)
		return value

	def value_to_string(self, obj):
		value = self.value_from_object(obj)
		return self.get_prep_value(value)



# -*- coding: utf-8 -*-
import types

from django.utils.encoding import escape_uri_path

from .settings import DUMMY_PARAMETER_NAMES


def get_clean_path(self):
	if hasattr(self, '_clean_path'):
		return getattr(self, '_clean_path')
	get = self.GET.copy()
	for param in DUMMY_PARAMETER_NAMES:
		get.pop(param, None)
	clean_path = '%s%s' % (
		escape_uri_path(self.path),
		('?' + get.urlencode() if get else '')
	)
	return clean_path


class RemoveDummyParametersMiddleware(object):
	def __init__(self, get_response):
		self.get_response = get_response

	def __call__(self, request):
		request = self.process_request(request) or request
		return self.get_response(request)

	def process_request(self, request):
		request.GET._mutable = True
		for param in DUMMY_PARAMETER_NAMES:
			request.GET.pop(param, None)
		request.GET._mutable = False
		request.get_clean_path = types.MethodType(get_clean_path, request)



# -*- coding: utf-8 -*-
from django import template
from django.core.exceptions import ImproperlyConfigured
from django_ajax_utils.pjax import is_pjax


register = template.Library()


class PjaxBlockNode(template.Node):
	def __init__(self, block_name, nodelist):
		self.block_name = block_name
		self.nodelist = nodelist

	def render(self, context):
		output = self.nodelist.render(context)
		if not 'request' in context:
			return output
		request = context['request']
		if is_pjax(request):
			if not hasattr(request, '_pjax_holders'):
				raise ImproperlyConfigured("Middleware django_ajax_utils.pjax.Middleware not enabled")
			request._pjax_holders.setdefault(self.block_name, [])
			request._pjax_holders[self.block_name].append(output)
			return ''
		return output


@register.tag
def pjaxblock(parser, token):
	try:
		tag_name, block_name = token.split_contents()
	except ValueError:
		raise template.TemplateSyntaxError("%r tag requires exactly two arguments" % token.contents.split()[0])
	nodelist = parser.parse(('end' + tag_name,))
	parser.delete_first_token()
	return PjaxBlockNode(block_name, nodelist)



# -*- coding: utf-8 -*-
import contextvars as cv
import json
import os
import re
import weakref
from importlib import import_module

from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from django.http.response import HttpResponse
from django.template import TemplateDoesNotExist
from django.template.loaders.base import Loader as BaseLoader
from django.utils.encoding import force_str

from .utility import get_accept_parser


def get_absolute_name(path):
	module_name, class_name = path.rsplit('.', 1)
	module = import_module(module_name)
	try:
		cls = getattr(module, class_name)
	except AttributeError:
		raise ImportError("cannot import name '%s'" % path)
	return cls


_current_request = None
get_current_request = getattr(settings, 'GET_CURRENT_REQUEST_CALLBACK', None)
if get_current_request is None:
	_current_request = cv.ContextVar("request", default=None)
	def get_current_request():
		return _current_request.get()
else:
	if isinstance(get_current_request, str):
		get_current_request = get_absolute_name(get_current_request)
_pjax_cache = {'include': None, 'exclude': None}


def pjax_supported(request):
	pjax_supported = getattr(request, 'pjax_supported', None)
	if pjax_supported is not None:
		return pjax_supported
	if _pjax_cache['include'] is None:
		_pjax_cache['include'] = [
			re.compile(pattern) for pattern in getattr(settings, 'PJAX_INCLUDE_URLPATTERNS', [])
		]
	if _pjax_cache['exclude'] is None:
		_pjax_cache['exclude'] = [
			re.compile(pattern) for pattern in getattr(settings, 'PJAX_EXCLUDE_URLPATTERNS', [])
		]
	if request.resolver_match is None:
		return False
	view_name = request.resolver_match.view_name
	is_included = False
	is_excluded = False
	for pattern in _pjax_cache['include']: # pylint: disable=not-an-iterable
		if pattern.match(view_name):
			is_included = True
			break
	for pattern in _pjax_cache['exclude']: # pylint: disable=not-an-iterable
		if pattern.match(view_name):
			is_excluded = True
			break
	pjax_supported = is_included and not is_excluded
	setattr(request, 'pjax_supported', pjax_supported)
	return getattr(request, 'pjax_supported')


def is_pjax_request(request):
	if not request:
		return False
	return get_accept_parser(request).has_content_type('application/pjax.json')


def is_pjax(request):
	return is_pjax_request(request) and pjax_supported(request)


def init_pjax_request(request):
	holders = {}
	setattr(request, '_pjax_holders', holders)
	return holders


def get_pjax_holders(request):
	return getattr(request, '_pjax_holders', {})


class Middleware(object):
	def __init__(self, get_response):
		self.get_response = get_response

	def __call__(self, request):
		if _current_request is not None:
			_current_request.set(request)
		init_pjax_request(request)
		response = self.get_response(request)
		if is_pjax(request):
			if response.status_code == 200:
				if not response.get('Content-Type', '').startswith('text/html'):
					return response
				pjax_holders = get_pjax_holders(request)
				blocks = {}
				for block_name, content in pjax_holders.items():
					blocks[block_name] = ''.join(content)

				json_data = {
					'is_pjax': True,
					'content': force_str(response.getvalue()),
					'blocks': blocks,
				}
				response = HttpResponse(json.dumps(json_data).encode('utf-8'), content_type="application/json")

		if is_pjax_request(request):
			if response.status_code in (301, 302):
				json_data = {
					'is_pjax': True,
					'plain_redirect': not pjax_supported(request),
					'redirect': response.url,
				}
				response = HttpResponse(json.dumps(json_data).encode('utf-8'), content_type="application/json")

		return response


class Loader(BaseLoader):
	@property
	def other_template_loaders(self):
		for template_loader in self.engine.template_loaders:
			if template_loader != self:
				yield template_loader

	def pjax_template_name(self, template_name):
		return '_pjax'.join(os.path.splitext(template_name))

	def get_template_sources(self, template_name, *args, **kwargs):
		sources = []
		if is_pjax(get_current_request()):
			pjax_template = self.pjax_template_name(template_name)
			for template_loader in self.other_template_loaders:
				sources += template_loader.get_template_sources(pjax_template, *args, **kwargs)
		for template_loader in self.other_template_loaders:
			sources += template_loader.get_template_sources(template_name, *args, **kwargs)
		return sources

	def get_contents(self, origin):
		return origin.loader.get_contents(origin)

	def load_template_source(self, template_name, template_dirs=None):
		if is_pjax(get_current_request()):
			try:
				return self.direct_load_template(self.pjax_template_name(template_name), template_dirs)
			except TemplateDoesNotExist:
				return self.direct_load_template(template_name, template_dirs)
		else:
			return self.direct_load_template(template_name, template_dirs)

	def direct_load_template(self, template_name, template_dirs):
		for template_loader in self.other_template_loaders:
			try:
				return template_loader.load_template_source(template_name, template_dirs)
			except TemplateDoesNotExist:
				pass
			except NotImplementedError:
				pass
			except AttributeError:
				pass
		raise TemplateDoesNotExist(template_name)


try:
	from jinja2 import nodes
	from jinja2.ext import Extension
	import jinja2
	import jinja2.exceptions

	class Environment(jinja2.Environment):
		def __init__(self, *args, **kwargs):
			super().__init__(*args, **kwargs)
			if jinja2.__version__.split('.')[0] == '3':
				self._load_template_impl = self._load_template3
			else:
				self._load_template_impl = self._load_template2

		def pjax_template_name(self, template_name):
			return '_pjax'.join(os.path.splitext(template_name))

		def check_pjax(self, name, request): # pylint: disable=unused-argument
			return is_pjax(request)

		def _load_template(self, name, globals): # pylint: disable=redefined-builtin
			return self._load_template_impl(name, globals)

		def _load_template3(self, name, globals): # pylint: disable=redefined-builtin
			if self.loader is None:
				raise TypeError("no loader for this environment specified")
			cache_key = (weakref.ref(self.loader), name)
			if self.cache is not None:
				template = self.cache.get(cache_key)
				if template is not None and (not self.auto_reload or template.is_up_to_date):
					if template == '':
						raise jinja2.exceptions.TemplateNotFound(name)
					if globals:
						template.globals.update(globals)
					return template
			try:
				template = self.loader.load(self, name, self.make_globals(globals))
			except jinja2.exceptions.TemplateNotFound:
				if self.cache is not None and not self.auto_reload:
					self.cache[cache_key] = ''
				raise

			if self.cache is not None:
				self.cache[cache_key] = template
			return template

		def _load_template2(self, name, globals): # pylint: disable=redefined-builtin
			if self.loader is None:
				raise TypeError("no loader for this environment specified")
			cache_key = (weakref.ref(self.loader), name)
			if self.cache is not None:
				template = self.cache.get(cache_key)
				if template is not None and (not self.auto_reload or template.is_up_to_date):
					if template == '':
						raise jinja2.exceptions.TemplateNotFound(name)
					return template
			try:
				template = self.loader.load(self, name, globals)
			except jinja2.exceptions.TemplateNotFound:
				if self.cache is not None and not self.auto_reload:
					self.cache[cache_key] = ''
				raise

			if self.cache is not None:
				self.cache[cache_key] = template
			return template

		def get_template(self, name, *args, **kwargs):
			if self.check_pjax(name, get_current_request()):
				try:
					return super().get_template(self.pjax_template_name(name), *args, **kwargs)
				except jinja2.exceptions.TemplateNotFound:
					return super().get_template(name, *args, **kwargs)
			else:
				return super().get_template(name, *args, **kwargs)


	class PjaxBlockExtension(Extension):
		tags = set(['pjaxblock'])

		def parse(self, parser):
			lineno = next(parser.stream).lineno
			namearg = parser.parse_expression()
			if isinstance(namearg, nodes.Name):
				namearg = nodes.Const(namearg.name)
			ctx_ref = jinja2.nodes.ContextReference()
			args = [ctx_ref, namearg]
			body = parser.parse_statements(['name:endpjaxblock'], drop_needle=True)
			return nodes.CallBlock(self.call_method('_process', args), [], [], body).set_lineno(lineno)

		def _process(self, context, name, caller):
			output = caller()
			if not 'request' in context:
				return output
			request = context['request']
			if is_pjax(request):
				if not hasattr(request, '_pjax_holders'):
					raise ImproperlyConfigured("Middleware django_ajax_utils.pjax.Middleware not enabled")
				request._pjax_holders.setdefault(name, [])
				request._pjax_holders[name].append(output)
				return ''
			return output


except ImportError:
	pass



# -*- coding: utf-8 -*-
import json
import xml.sax
from io import BytesIO, StringIO

from django.utils.encoding import force_bytes


encoder = json.JSONEncoder()


class CompilerHandler(xml.sax.ContentHandler):
	INDENT = '\t'

	def __init__(self, *args, **kwargs):
		super(CompilerHandler, self).__init__(*args, **kwargs)
		self.output = StringIO()
		self.depth = -1
		self.lastDepth = -2
		self.textContent = None

	def startElement(self, name, attrs):
		self.object_start()
		if name[0] == ':':
			self.write_indent()
			self.output.write('E(' + encoder.encode('#text=' + name[1:]) + ', ')
			self.textContent = ''
		else:
			name = name.replace(':', '=')
			self.write_indent()
			self.output.write('E(' + encoder.encode(name) + ', [')
			for name, value in attrs.items():
				name = name.replace(':', '=')
				self.object_start()
				self.write_indent()
				self.output.write('E(' + encoder.encode('@' + name) + ', ' + encoder.encode(value) + ')')
				self.object_end()

	def endElement(self, name):
		if self.textContent is not None:
			self.output.write(encoder.encode(self.textContent))
			self.textContent = None
		if name[0] == ':':
			self.output.write(')')
		else:
			self.output.write('\n')
			self.write_indent()
			self.output.write('])')
		self.object_end()

	def characters(self, content):
		if not content.strip():
			return
		if self.textContent is not None:
			self.textContent += content
		else:
			self.object_start()
			self.write_indent()
			self.output.write('E("#text", ' + encoder.encode(content) + ')')
			self.object_end()

	def object_start(self):
		if self.lastDepth == self.depth:
			self.output.write(',')
		self.output.write('\n')
		self.lastDepth = self.depth
		self.depth += 1

	def object_end(self):
		self.depth -= 1
		self.lastDepth = self.depth

	def write_indent(self):
		self.output.write(self.INDENT * self.depth)

	def get_output(self):
		return self.output.getvalue()


def compile(source):
	handler = CompilerHandler()
	parser = xml.sax.make_parser()
	parser.setContentHandler(handler)
	parser.setErrorHandler(xml.sax.ErrorHandler())
	parser.parse(BytesIO(force_bytes(source)))
	return handler.get_output()



# -*- coding: utf-8 -*-
import json

from django import template, urls
from django.utils.encoding import force_str
from django.utils.safestring import mark_safe


register = template.Library()
url_resolver = urls.get_resolver()


def prepare_url_list(resolver, namespace_path='', namespace=''):
	for url_name in resolver.reverse_dict.keys():
		if not isinstance(url_name, str):
			continue
		url_name = force_str(url_name)
		formated_patterns = []
		for url_pattern in resolver.reverse_dict.getlist(url_name):
			for url_format, url_params in url_pattern[0]:
				formated_patterns.append({'pattern': namespace_path + url_format, 'params': url_params})
		yield {'name': namespace + url_name, 'patterns': formated_patterns}

	for inner_ns, (inner_ns_path, inner_resolver) in resolver.namespace_dict.items():
		inner_ns_path = namespace_path + inner_ns_path
		inner_ns = namespace + inner_ns + ':'

		if inner_ns_path:
			inner_resolver = urls.get_ns_resolver(inner_ns_path, inner_resolver, ())
			inner_ns_path = ''

		for url_pattern in prepare_url_list(inner_resolver, inner_ns_path, inner_ns):
			yield url_pattern


def safe_json_str(json_str):
	# from https://gist.github.com/amacneil/5af7cd0e934f5465b695
	unsafe_chars = {
		'&': '\\u0026',
		'<': '\\u003c',
		'>': '\\u003e',
		'\u2028': '\\u2028',
		'\u2029': '\\u2029'
	}
	for (c, d) in unsafe_chars.items():
		json_str = json_str.replace(c, d)
	return mark_safe(json_str)


@register.simple_tag
def js_urlpatterns(*export_patterns):
	if len(export_patterns) == 1 and isinstance(export_patterns[0], (list, tuple)):
		export_patterns = export_patterns[0]
	export_patterns = frozenset(export_patterns)
	if export_patterns in js_urlpatterns.cache:
		return js_urlpatterns.cache[export_patterns]
	if export_patterns:
		url_patterns = {pattern['name']: pattern['patterns'] for pattern in prepare_url_list(url_resolver) if pattern['name'] in export_patterns}
	else:
		url_patterns = {pattern['name']: pattern['patterns'] for pattern in prepare_url_list(url_resolver)}
	js_urlpatterns.cache[export_patterns] = safe_json_str(json.dumps(url_patterns))
	return js_urlpatterns.cache[export_patterns]
js_urlpatterns.cache = {}


try:
	from django_jinja import library

	library.global_function(js_urlpatterns)
except ImportError:
	pass



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.core.exceptions import SuspiciousOperation, ImproperlyConfigured
from django.http.response import HttpResponseRedirect
from django.views.generic.base import ContextMixin, TemplateResponseMixin
from django.views.generic.edit import ProcessFormView


class MultiFormMixin(ContextMixin):
	"""
	Dictionary of form classes, eg.

	{
		'login': LoginForm,
		'sugnup': SignupForm
	}
	"""
	form_classes = {}
	"""
	Process forms in groups, eg.

	{
		'address': ['shipping_address', 'invoicing_address']
	}
	"""
	grouped_forms = {}

	"""
	Custom prefixes for selected forms

	{
		'login': '', # no-prefix
		'signup': 'signup-',
	}
	"""
	prefixes = {}

	"""
	Allow only these actions:

	individual - submit individual forms with POST[action] = form name
	grouped - submit form groups with POST[action] = group name
	all - submit all forms without action parameter
	"""
	allow_partials = {'individual', 'grouped', 'all'}

	initial = {}
	success_url = None

	def get_form_classes(self):
		return self.form_classes

	def get_forms(self, form_classes, form_names):
		forms = {}
		for form_name, form_class in form_classes.items():
			bind = form_name in form_names
			forms[form_name] = self._create_form(form_name, form_class, bind)
		return forms

	def get_form_kwargs(self, form_name, bind_form=False):
		kwargs = {}
		kwargs.update({'initial':self.get_initial(form_name)})
		kwargs.update({'prefix':self.get_prefix(form_name)})

		if bind_form:
			kwargs.update(self._bind_form_data())

		form_kwargs_method = 'get_%s_form_kwargs' % form_name
		if hasattr(self, form_kwargs_method):
			kwargs.update(getattr(self, form_kwargs_method)())

		return kwargs

	def forms_valid(self, forms):
		for form_name, form_instance in forms.items():
			if form_instance.is_bound:
				form_valid_method = '%s_form_valid' % form_name
				if hasattr(self, form_valid_method):
					return getattr(self, form_valid_method)(form_instance)
		return HttpResponseRedirect(self.get_success_url())

	def forms_invalid(self, forms):
		return self.render_to_response(self.get_context_data(**forms))

	def get_initial(self, form_name):
		initial_method = 'get_%s_initial' % form_name
		if hasattr(self, initial_method):
			return getattr(self, initial_method)()
		else:
			return self.initial.copy()

	def get_prefix(self, form_name):
		return self.prefixes.get(form_name, form_name)

	def get_success_url(self):
		if self.success_url is None:
			raise ImproperlyConfigured("No success_url defined")
		return self.success_url

	def _create_form(self, form_name, cls, bind_form):
		form_kwargs = self.get_form_kwargs(form_name, bind_form)
		form_create_method = 'create_%s_form' % form_name
		if hasattr(self, form_create_method):
			form = getattr(self, form_create_method)(**form_kwargs)
		else:
			form = cls(**form_kwargs)
		return form

	def _bind_form_data(self):
		if self.request.method == 'POST':
			return {'data': self.request.POST, 'files': self.request.FILES}
		return {}


class ProcessMultiFormsMixin(ProcessFormView):
	def get(self, request, *args, **kwargs):
		form_classes = self.get_form_classes()
		forms = self.get_forms(form_classes, [])
		return self.render_to_response(self.get_context_data(**forms))

	def post(self, request, *args, **kwargs):
		form_classes = self.get_form_classes()
		form_names = self.get_form_names_for_action(self.get_action())
		forms = self.get_forms(form_classes, form_names)
		if all([forms[form_name].is_valid() for form_name in form_names]):
			return self.forms_valid(forms)
		else:
			return self.forms_invalid(forms)

	def get_action(self):
		return self.request.POST.get('action')

	def get_form_names_for_action(self, action):
		if not action:
			if 'all' in self.allow_partials:
				return self.form_classes.keys()
			else:
				raise SuspiciousOperation("Action not allowed")
		if 'grouped' in self.allow_partials:
			if action in self.grouped_forms:
				return self.grouped_forms[action]
		if 'individual' in self.allow_partials:
			if action in self.form_classes:
				return (action,)
		raise SuspiciousOperation("Action not allowed")


class BaseMultiFormsView(MultiFormMixin, ProcessMultiFormsMixin):
	pass


class MultiFormsView(TemplateResponseMixin, BaseMultiFormsView):
	pass



# -*- coding: utf-8 -*-
from django.http.response import HttpResponseRedirect, HttpResponsePermanentRedirect


class PlainRedirect(object):
	plain_redirect = True


class PlainHttpResponseRedirect(PlainRedirect, HttpResponseRedirect):
	pass


class PlainHttpResponsePermanentRedirect(PlainRedirect, HttpResponsePermanentRedirect):
	pass



#!/usr/bin/env python
import os
import sys


if __name__ == "__main__":
	sys.path.append(os.path.join(os.path.dirname(__file__), '..'))
	os.environ.setdefault("DJANGO_SETTINGS_MODULE", "web.settings")
	try:
		from django.core.management import execute_from_command_line
	except ImportError:
		# The above import may fail for some other reason. Ensure that the
		# issue is really that Django is missing to avoid masking other
		# exceptions on Python 2.
		try:
			import django
		except ImportError:
			raise ImportError(
				"Couldn't import Django. Are you sure it's installed and "
				"available on your PYTHONPATH environment variable? Did you "
				"forget to activate a virtual environment?"
			)
		raise
	execute_from_command_line(sys.argv)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.contrib import messages
from django.http.response import HttpResponseRedirect
from django.urls import reverse
from django.views.generic import TemplateView, FormView

from .forms import SignupForm, SimpleForm, MessagesForm, SimpleFormSet
from django_ajax_utils import multiforms
from django_ajax_utils.views import AjaxFormMixin


class AjaxFormView(AjaxFormMixin, FormView):
	form_class = SignupForm

	def form_valid(self, form):
		if not self.only_validate_form:
			return HttpResponseRedirect(reverse('home'))
		return super(AjaxFormView, self).form_valid(form)


class FormsetView(AjaxFormMixin, FormView):
	form_class = SimpleFormSet

	def form_valid(self, form):
		if not self.only_validate_form:
			return HttpResponseRedirect(reverse('home'))
		return super(FormsetView, self).form_valid(form)


class MessagesView(FormView):
	form_class = MessagesForm

	def get(self, request, *args, **kwargs):
		messages.info(self.request, "Test message")
		return super(MessagesView, self).get(request, *args, **kwargs)

	def form_valid(self, form):
		data = form.cleaned_data
		messages.add_message(self.request, level=data['level'], message=data['message'])
		return self.render_to_response(self.get_context_data(form=form))


class PjaxMessagesView(TemplateView):
	def get(self, request, *args, **kwargs):
		messages.info(self.request, "Test message")
		return super(PjaxMessagesView, self).get(request, *args, **kwargs)


class PjaxFormPostView(AjaxFormMixin, FormView):
	form_class = MessagesForm

	def form_valid(self, form):
		if self.only_validate_form:
			return self.render_to_response(self.get_context_data(form=form))
		data = form.cleaned_data
		messages.add_message(self.request, level=data['level'], message=data['message'])
		return HttpResponseRedirect(self.request.path)


class PjaxFormGetView(AjaxFormMixin, TemplateView):
	def get_context_data(self, **kwargs):
		ctx = super(PjaxFormGetView, self).get_context_data(**kwargs)
		ctx['form'] = MessagesForm(self.request.GET or None)
		return ctx


class MultiFormsView(multiforms.MultiFormsView):
	form_classes = {
		'form1': SimpleForm,
		'form2': SimpleForm,
		'form3': SimpleForm,
	}
	grouped_forms = {
		'group1': ['form2', 'form3']
	}

	def forms_valid(self, forms):
		if not self.only_validate_form:
			return HttpResponseRedirect(reverse('home'))
		return super(MultiFormsView, self).forms_valid(forms)

	def get_form1_initial(self):
		return {'text': 'initial'}


home_view = TemplateView.as_view(template_name='home.html')
utils_view = TemplateView.as_view(template_name='utils.html')
ajaxform_view = AjaxFormView.as_view(template_name='ajaxform.html')
messages_view = MessagesView.as_view(template_name='messages.html')
pjax_view = TemplateView.as_view(template_name='pjax.html')
pjax_messages_view = PjaxMessagesView.as_view(template_name='pjax_messages.html')
pjax_form_post_view = PjaxFormPostView.as_view(template_name='pjax_form_post.html')
pjax_form_get_view = PjaxFormGetView.as_view(template_name='pjax_form_get.html')
urlpatterns_view = TemplateView.as_view(template_name='urlpatterns.html')
form_utils_view = AjaxFormView.as_view(template_name='form_utils/base.html')
form_utils_foundation_view = AjaxFormView.as_view(template_name='form_utils/foundation.html')
formset_view = FormsetView.as_view(template_name='form_utils/formset.html')
multiforms_view = MultiFormsView.as_view(template_name='form_utils/multiforms.html')



# -*- coding: utf-8 -*-
from django.conf import settings


DUMMY_PARAMETER_NAMES = getattr(settings, "DUMMY_PARAMETER_NAMES", ['_dummy'])
FORMUTILS_ROW_TEMPLATE_ATTRIBUTE = '_formrow_template'
FORMUTILS_DEFAULT_ROW_TEMPLATE = getattr(settings, "FORMUTILS_DEFAULT_ROW_TEMPLATE", None)
FORMUTILS_DEFAULT_LAYOUT_TEMPLATE = getattr(settings, "FORMUTILS_DEFAULT_LAYOUT_TEMPLATE", None)
FORMUTILS_DEFAULT_ROW_TEMPLATE_GET = getattr(settings, "FORMUTILS_DEFAULT_ROW_TEMPLATE_GET", None)
FORMUTILS_DEFAULT_LAYOUT_TEMPLATE_GET = getattr(settings, "FORMUTILS_DEFAULT_LAYOUT_TEMPLATE_GET", None)
GET_CURRENT_REQUEST_CALLBACK = getattr(settings, "GET_CURRENT_REQUEST_CALLBACK", None)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.conf.urls import url

from . import views


urlpatterns = [
	url(r'^$', views.home_view, name='home'),
	url(r'^utils/$', views.utils_view, name='utils'),
	url(r'^ajaxform/$', views.ajaxform_view, name='ajaxform'),
	url(r'^messages/$', views.messages_view, name='messages'),
	url(r'^pjax/$', views.pjax_view, name='pjax'),
	url(r'^pjax/messages/$', views.pjax_messages_view, name='pjax_messages'),
	url(r'^pjax/form/post/$', views.pjax_form_post_view, name='pjax_form_post'),
	url(r'^pjax/form/get/$', views.pjax_form_get_view, name='pjax_form_get'),
	url(r'^urlpatterns/(?:(?P<param>\w+)/)?$', views.urlpatterns_view, name='urlpatterns'),
	url(r'^form-utils/$', views.form_utils_view, name='form_utils'),
	url(r'^form-utils/foundation/$', views.form_utils_foundation_view, name='form_utils_foundation'),
	url(r'^form-utils/fomset/$', views.formset_view, name='formset'),
	url(r'^form-utils/multiforms/$', views.multiforms_view, name='multiforms'),
]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django import forms
from django.contrib.messages import constants
from django.core import validators
from django.forms import formset_factory
from django.utils.translation import ugettext_lazy as _

from django_ajax_utils.forms import AutoPlaceholderFormMixin, SetWidgetAttrsMixin


# Hack for disable required html attribute
class TextInput(forms.TextInput):
	def use_required_attribute(self, initial):
		return False


class PasswordInput(forms.PasswordInput):
	def use_required_attribute(self, initial):
		return False


class SignupForm(AutoPlaceholderFormMixin, SetWidgetAttrsMixin, forms.Form):
	widget_attrs = {
		'username': {'maxlength': 10},
	}

	username = forms.CharField(label=_("Username"), widget=TextInput)
	password1 = forms.CharField(label=_("Password"), widget=PasswordInput, validators=[validators.MinLengthValidator(3)])
	password2 = forms.CharField(label=_("Password confirmation"), widget=PasswordInput)

	def clean(self):
		cleaned_data = super(SignupForm, self).clean()
		password1 = cleaned_data.get('password1')
		password2 = cleaned_data.get('password2')

		if password1 and password2 and password1 != password2:
			raise forms.ValidationError(_("The two password fields didn't match."))


class MessagesForm(forms.Form):
	LEVEL_CHOICES = (
		(constants.DEBUG, "Debug"),
		(constants.INFO, "Info"),
		(constants.SUCCESS, "Success"),
		(constants.WARNING, "Warning"),
		(constants.ERROR, "Error"),
	)

	message = forms.CharField()
	level = forms.ChoiceField(choices=LEVEL_CHOICES, initial=constants.INFO)


class SimpleForm(forms.Form):
	text = forms.CharField(widget=TextInput)


SimpleFormSet = formset_factory(SimpleForm, extra=1, min_num=2, validate_min=True)
